use std::io;

#[derive(Debug, PartialOrd, PartialEq)]
#[allow(non_camel_case_types)]
pub enum NodeManagementError {
    /// recent packet was broken; this packet's data did not arrive properly.
    net_broken_packet        = 0x00,
    /// the packet sent is not valid; this packet's data does not make sense.
    net_invalid_packet       = 0x01,
    /// the destination could not be reached.
    net_dest_unreachable     = 0x02,
    
    /// `packet_id` was used recently.
    mgmt_duplicate_packet    = 0x10,
    /// the addr requested is already in use.
    mgmt_addr_in_use         = 0x11,
    /// the src addr is not registered with the master node.
    mgmt_not_registered      = 0x12,
    /// the sent packet could not be delivered.
    mgmt_not_delivered       = 0x13,

    //// the property specified is invalid.
    node_invalid_property    = 0x20,
    /// the request could not be completed.
    node_failed_request      = 0x21,
}

impl Into<u8> for NodeManagementError {
    fn into(self) -> u8 {
        self as u8
    }
}

impl TryFrom<u8> for NodeManagementError {
    type Error = io::Error;
    fn try_from(value: u8) -> io::Result<Self> {
        match value {
            0x00 => Ok(NodeManagementError::net_broken_packet),
            0x01 => Ok(NodeManagementError::net_invalid_packet),
            0x02 => Ok(NodeManagementError::net_dest_unreachable),

            0x10 => Ok(NodeManagementError::mgmt_duplicate_packet),
            0x11 => Ok(NodeManagementError::mgmt_addr_in_use),
            0x12 => Ok(NodeManagementError::mgmt_not_registered),
            0x13 => Ok(NodeManagementError::mgmt_not_delivered),
            _ => Err(io::Error::new(io::ErrorKind::InvalidInput, "non supported error"))
        }
    }
}